#!/bin/sh
####
#
# SuSE master control security mechanism for the daily/weekly/monthly
# security checks, by Marc Heuse <marc@suse.de>, version 2.0
#
####
VERSION="v2.1de"

test -e /etc/rc.config || touch /etc/rc.config	# prevent errors on non-SuSE
. /etc/rc.config
test -z "$SECCHK_USER" && SECCHK_USER="root"

function syntax () {
    /bin/echo "Syntax: $0 "'daily|weekly|monthly'
    exit 1
}
function disclaimer () {
    /bin/echo
    /bin/echo "DISCLAIMER"
    /bin/echo
    /bin/echo "Bitte beachten Sie, dass diese Security-Checks nie komplett oder endgueltig sicher sein werden."
    /bin/echo "Jeder Angreifer, der einmal \"root\"-Rechte auf ihrem System erlangt hat, kann JEDEN Sicherheitstest"
    /bin/echo "umgehen!"
    /bin/echo
}

test -z "$1" && syntax
. /etc/rc.config

export PATH="/sbin:/usr/sbin:/bin:/usr/bin"
umask 077
test -x "/usr/sbin/sendmail" && MAILER="/usr/sbin/sendmail"
test -z "$MAILER" && test -x "/usr/bin/mailx" && MAILER="/usr/bin/mailx"
test -z "$MAILER" && test -x "/usr/lib/sendmail" && MAILER="/usr/lib/sendmail"
test -z "$MAILER" && echo "Kann kein Email-Programm finden!"
test -z "$MAILER" && exit 1
test -z "$SEC_BIN" && SEC_BIN="/usr/lib/secchk"
test -z "$SEC_DATA" && SEC_VAR="/var/lib/secchk"
export MAILER
SEC_DATA="$SEC_VAR/data"
OUT1="$SEC_VAR/security-report-daily.new"
OLD1="$SEC_VAR/security-report-daily"
OUT2="$SEC_VAR/security-report-weekly.new"
OLD2="$SEC_VAR/security-report-weekly"
OLD3="$SEC_VAR/security-report-monthly"
if [ ! -d "$SEC_VAR" ]; then
    rm -rf "$SEC_VAR"
    mkdir "$SEC_VAR" || exit 1
fi
if [ ! -d "$SEC_DATA" ]; then
    rm -rf "$SEC_DATA"
    mkdir "$SEC_DATA" || exit 1
fi
for i in "$OLD1" "$OLD2" "$OLD3" ; do
    if [ "$i" != "" ]; then
        if [ ! -e "$i" ]; then
   	    touch "$i"
        fi
    fi
done

test "$1" = "daily" && (
 /bin/sh "$SEC_BIN/security-daily.sh" 1> "$OUT1"
 /usr/bin/diff -q -w "$OLD1" "$OUT1" 1> /dev/null || (
    {
      echo "To: $SECCHK_USER"
      echo -e "Subject: Tages-Sicherheitsbericht fuer `hostname`: Aenderungen\n"
      echo "Tages-Sicherheitscheck $VERSION von Marc Heuse <marc@suse.de>"
      echo "Dies ist eine automatisch generierte Email vom Sicherheits-Tool. Wenn Sie diese Nachricht abschalten"
      echo "moechten, geben Sie bitte \"mv /etc/cron.d/seccheck /etc/cron.d_seccheck.save\" auf der Konsole ein."
      disclaimer
      echo -e "Aenderungen auf Ihrem System `hostname`:\n"
      /usr/bin/diff -u -w "$OLD1" "$OUT1" | sed 's/^@@.*/\
* Aenderungen (+: neue Eintraege, -: geloeschte Eintraege):\
	/' | egrep '^[+*-]|^$' |sed 's/^+++/NEU:/' | sed 's/^---/ALT:/' | sed 's/^[+-]/& /'
    } | $MAILER "$SECCHK_USER"
    /bin/mv "$OUT1" "$OLD1"
 )
 rm -f "$OUT1"
)

test "$1" = "weekly" && (
 /bin/sh "$SEC_BIN/security-weekly.sh" 1> "$OUT2"
 if [ -s "$OUT2" ]; then
    {
      echo "To: $SECCHK_USER"
      echo -e "Subject: Woechentlicher Sicherheitsbericht fuer `hostname`: Aenderungen\n"
      echo "Woechentlicher Sicherheitscheck $VERSION von Marc Heuse <marc@suse.de>"
      echo "Dies ist eine automatisch generierte Email vom Sicherheits-Tool. Wenn Sie diese Nachricht abschalten"
      echo "moechten, geben Sie bitte \"mv /etc/cron.d/seccheck /etc/cron.d_seccheck.save\" auf der Konsole ein."
      disclaimer
      echo -e "Aenderungen auf Ihrem System `hostname`:\n"
      cat "$OUT2"
    } | $MAILER "$SECCHK_USER"
    mv "$OUT2" "$OLD2"
 fi
 rm -f "$OUT2"
)

test "$1" = "monthly" && (
 test -s "$OLD1" || /bin/sh "$SEC_BIN/security-daily.sh" 1> "$OLD1"
 test -e "$SEC_DATA/devices" || /bin/sh "$SEC_BIN/security-weekly.sh" 1> "$OLD2"
 {
      echo "To: $SECCHK_USER"
      echo -e "Subject: Monatlicher Sicherheitsbericht fuer `hostname`: Komplett\n"
      echo "Monatlicher Sicherheitscheck $VERSION von Marc Heuse <marc@suse.de>"
      echo "Dies ist eine automatisch generierte Email vom Sicherheits-Tool. Wenn Sie diese Nachricht abschalten"
      echo "moechten, geben Sie bitte \"mv /etc/cron.d/seccheck /etc/cron.d_seccheck.save\" auf der Konsole ein."
      disclaimer
      echo -e "Komplettes Monatslisting von `hostname`:\n"
      /bin/sh "$SEC_BIN/security-monthly.sh"
 } | tee "$OLD3" | $MAILER "$SECCHK_USER"
)

exit 0
