#!/bin/sh
####
#
# SuSE daily security check v2.0 by Marc Heuse <marc@suse.de>
#
# most code was ripped from the OpenBSD /etc/security script ;-)
#
#
# Geaendert und uebersetzt ins Deutsche von Lars Rupp <L.Rupp@web.de>
#
####
#
# TODO: maybe directory writable checks for jobs run in crontab
#
PATH=/sbin:/usr/sbin:/bin:/usr/bin
umask 077

TMPDIR=`/bin/mktemp -d /tmp/.security.XXXXXX` || {
    TMPDIR="/tmp/.security-daily.sh.redhatshouldupdatetheirmktemp.$$"
    rm -rf "$TMPDIR"
    mkdir "$TMPDIR" || exit 1
}
trap 'rm -rf $TMPDIR; exit 1' 0 1 2 3 13 15
LOG="$TMPDIR/security.log"
ERR="$TMPDIR/security.err"
OUT="$TMPDIR/security.out"
TMP1="$TMPDIR/security.tmp1"
TMP2="$TMPDIR/security.tmp2"

#
# /etc/passwd check
#
PW="/etc/passwd"
awk -F: '{
        if ($0 ~ /^[ 	]*$/) {
                printf("Zeile %d enthaelt keine Daten.\n", NR);
                next;
        }
        if (NF != 7)
                printf("Zeile %d hat eine falsche Anzahl an Feldern.\n", NR);
        if ($1 ~ /^[+-]$/)
                next;
        if ($1 == "")
                printf("Zeile %d hat ein leeres Login-Feld.\n", NR);
        else if ($1 !~ /^[A-Za-z0-9][A-Za-z0-9_\.-]*$/)
                printf("Der Login %s enthaelt Nicht-Alphanumerische Zeichen - z.B. Umlaute.\n", $1);
        if (length($1) > 12)
                printf("Der Login %s hat mehr als 12 Zeichen.\n", $1);
        if ($2 == "")
                printf("Der Login %s hat kein Passwort.\n", $1);
        else if ($2 !~ /^[x*!]+$/)
		printf("Der Login %s hat ein ungeschuetztes Passwort (steht im Klartext in der Datei).\n", $1);
#if ($2 != "" && length($2) != 13 && ($10 ~ /.*sh$/ || $10 == "") &&
#   ($2 !~ /^\$[0-9a-f]+\$/)) {
#       if (system("test -d "$9" -a ! -r "$9"") == 0)
#          printf("Login %s if off but still has valid shell and home directory is unreadable\n\t by root; cannot check for existance of alternate access files.\n", $1);
#       else if (system("for file in .ssh .rhosts .shosts .klogin;
#               do if test -e "$9"/$file; then if ((ls -ld "$9"/$file | cut -b 2-10 | grep -q r) && (test ! -O "$9"/$file)) ; then exit 1; fi; fi; done"))
#                  printf("Login %s is off but still has a valid shell and alternate access files in\n\t home directory are still readable.\n",$1);
#}
        if ($3 == 0 && $1 != "root")
                printf("Der Login %s hat die Nutzer-ID 0 und damit root-Rechte.\n", $1);
        if ($3 == 1 && $1 != "bin")
		printf("Der Login %s hat die Nutzer-ID 1 und damit globale Rechte.\n", $1);
        if ($3 < 0)
                printf("Der Login %s hat eine negative Nutzer-ID.\n", $1);
        if ($4 < 0)
                printf("Der Login %s hat eine negative Gruppen-ID.\n", $1);
	if ($4 == 0 && $1 != "root")
		printf("Der Login %s ist Mitglied der root-Gruppe.\n", $1);
	if ($4 == 1 && $1 != "bin")
		printf("Der Login %s ist Mitglied der binaeren-Gruppe.\n", $1);
}' < $PW > $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe die Datei $PW:\n"
        cat "$OUT"
fi
awk -F: '{ print $1 }' $PW | sort | uniq -d > $OUT
if [ -s "$OUT" ] ; then
        printf "\n$PW enthaelt doppelte Nutzer-Namen.\n"
        column "$OUT"
fi
awk -F: '{ print $1 " " $3 }' $PW | sort -n +1 | tee $TMP1 |
uniq -d -f 1 | awk '{ print $2 }' > $TMP2
if [ -s "$TMP2" ] ; then
        echo -e "\n$PW enthaelt doppelte Nutzer-IDs:"
        while read uid; do
                grep -w $uid $TMP1
        done < $TMP2 | column
fi
cp -pf $PW $PW.backup
#
# /etc/shadow check
#
PW="/etc/shadow"
awk -F: '{
        if ($0 ~ /^[ 	]*$/) {
                printf("Zeile %d enthaelt keinen Eintrag.\n", NR);
                next;
        }
        if (NF != 9)
                printf("Zeile %d hat eine falsche Anzahl an Feldern.\n", NR);
        if ($1 ~ /^[+-]$/)
                next;
        if ($1 == "")
                printf("Zeile %d hat ein leeres Login-Feld.\n", NR);
        else if ($1 !~ /^[A-Za-z0-9][A-Za-z0-9_-]*$/)
                printf("Der Login %s hat Nicht-Alphanumerische Zeichen - z.B. Umlaute.\n", $1);
        if (length($1) > 12)
                printf("Der Login %s hat mehr als 12 Zeichen.\n", $1);
        if ($2 == "")
                printf("Der Login %s hat kein Passwort.\n", $1);
	if ($2 != "" && length($2) != 13 && length($2) != 34 &&
	    length($2) != 1 && $2 !~ /^\$[0-9a-f]+\$/)
		printf("Der Login %s hat eine unbrauchbare Feldlaenge fuer das Passwort.\n", $1);
}' < $PW > "$OUT"
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe die Datei $PW :\n"
        cat "$OUT"
fi
awk -F: '{ print $1 }' $PW | sort | uniq -d > $OUT
if [ -s "$OUT" ] ; then
        printf "\n$PW enthaelt doppelte Nutzer-Namen.\n"
        column "$OUT"
fi
cp -fp "$PW" "$PW.backup"
#
# /etc/group checking
#
GRP=/etc/group
awk -F: '{
        if ($0 ~ /^[	 ]*$/) {
                printf("Zeile %d enthaelt keinen Eintrag.\n", NR);
                next;
        }
        if ($1 ~ /^[+-]$/)
                next;
        if (NF != 4)
                printf("zeile %d enthaelt eine ungueltige Anzahl an Feldern.\n", NR);
        if ($1 !~ /^[A-za-z0-9][A-za-z0-9_-]*$/)
                printf("Gruppe %s hat Nicht-Alphanumerische Zeichen, z.B. Umlaute.\n", $1);
        if (length($1) > 12)
                printf("Gruppe %s hat mehr als 12 Zeichen.\n", $1);
        if ($3 !~ /[0-9]*/)
                printf("Der Login %s hat eine negative Group-ID.\n", $1);
        if (length($4) > 0 && $3 < 3)
		printf("Gruppe %s(%s) hat die folgenden Mitglieder: %s\n", $1, $3, $4);
}' < $GRP > $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe die Datei $GRP :\n"
        cat "$OUT"
fi
awk -F: '{ print $1 }' $GRP | sort | uniq -d > $OUT
if [ -s "$OUT" ] ; then
        printf "\n$GRP enthaelt doppelte Gruppen-Namen.\n"
        column "$OUT"
fi
#
# checking root's login scrips for secure path and umask
#
> $OUT
> $TMP1
> $TMP2
rhome=/root
umaskset=no
list="/etc/csh.cshrc /etc/csh.login"
for i in $list ; do
        if [ -s "$i" ] ; then
                if egrep umask $i > /dev/null ; then
                        umaskset=yes
                fi
                egrep umask $i |
                awk '$2 % 100 < 20 \
                        { print "Root umask ist schreibbar fuer die Gruppe." }
                     $2 % 10 < 2 \
                        { print "Root umask ist weltweit schreibbar." }' >> $OUT
                SAVE_PATH=$PATH
                unset PATH 2> /dev/null || PATH="" # redhat ... 
                /bin/csh -f -s << end-of-csh > /dev/null 2>&1
                        test -f "$i" && (	# still a race
                            source $i
                            /bin/ls -ldcbg \$path > $TMP1
			)
end-of-csh
                PATH=$SAVE_PATH
                awk '{
                        if ($9 ~ /^\.$/) {
                                print "Der root Pfad enthaelt .";
                                next;
                        }
                     }
                     $1 ~ /^d....w/ \
        { print "Root Pfad-Verzeichnis " $9 " ist schreibbar fuer die Gruppe." } \
                     $1 ~ /^d.......w/ \
        { print "Root Pfad-Verzeichnis " $9 " ist weltweit schreibbar." }' \
                < $TMP1 >> $TMP2
        fi
done
if [ $umaskset = "no" -o -s "$TMP2" ] ; then
	sort -u $TMP2 > $OUT
        printf "\nUeberpruefe root csh paths, umask Werte:\n$list\n"
        if [ -s "$OUT" ] ; then
                cat "$OUT"
        fi
        if [ $umaskset = "no" ] ; then
                printf "\nRoot csh Startdateien setzen keine umask.\n"
        fi
fi
> $OUT
> $TMP1
> $TMP2
rhome=/root
umaskset=no
list="/etc/profile ${rhome}/.profile ${rhome}/.bashrc ${rhome}/.bash_login"
for i in $list; do
        if [ -s "$i" ] ; then
                if egrep umask $i > /dev/null ; then
                        umaskset=yes
                fi
                egrep umask $i |
                awk '$2 % 100 < 20 \
                        { print "Root umask ist schreibbar fuer die Gruppe" } \
                     $2 % 10 < 2 \
                        { print "Root umask ist weltweit schreibbar" }' >> $OUT
                SAVE_PATH=$PATH
                unset PATH 2> /dev/null || PATH="" # redhat again ...
                /bin/sh << end-of-sh > /dev/null 2>&1
                        file "$i" | grep -qw text && . $i
                        list=\`echo \$PATH | /usr/bin/sed -e 's/:/ /g'\`
			/bin/ls -ldgbT \$list > $TMP1
end-of-sh
                PATH=$SAVE_PATH
                awk '{
                        if ($9 ~ /^\.$/) {
                                print "Der root Pfad enthaelt .";
                                next;
                        }
                     }
                     $1 ~ /^d....w/ \
        { print "Root Pfad-Verzeichnis " $9 " ist schreibbar fuer die Gruppe." } \
                     $1 ~ /^d.......w/ \
        { print "Root Pfad-Verzeichnis " $9 " ist weltweit schreibbar." }' \
                < $TMP1 >> $TMP2

        fi
done
if [ $umaskset = "no" -o -s "$TMP2" ] ; then
	sort -u $TMP2 > $OUT
        printf "\nUeberpreufe root sh paths, umask Werte:\n$list\n"
        if [ -s "$OUT" ] ; then
                cat "$OUT"
        fi
        if [ $umaskset = "no" ] ; then
                printf "\nRoot sh Stardateien setzen keine umask.\n"
        fi
fi
#
# Misc. file checks
#
# root/uucp/bin/daemon etc. should be in /etc/ftpusers.
if [ -s /etc/ftpusers ]; then
	> $OUT
	grep -q '^root$' /etc/ftpusers || echo root >> $OUT
	grep -q '^bin$' /etc/ftpusers || echo bin >> $OUT
        grep -q '^uucp$' /etc/ftpusers || echo uucp >> $OUT
        grep -q '^daemon$' /etc/ftpusers || echo daemon >> $OUT
        grep -q '^nobody$' /etc/ftpusers || echo nobody >> $OUT
        grep -q '^lp$' /etc/ftpusers || echo lp >> $OUT
        grep -q '^man$' /etc/ftpusers || echo man >> $OUT
	if [ -s "$OUT" ] ; then
	    printf "\nFolgende System-Accounts fehlen in der Datei /etc/ftpusers:\n"
	    cat "$OUT"
	fi
fi
# executables should not be in the /etc/aliases file.
if [ -s /etc/aliases ]; then
    grep -v '^#' /etc/aliases | grep '|' > $OUT
    if [ -s "$OUT" ] ; then
            printf "\nDie folgenden Programme werden ueber /etc/aliases ausgefuehrt (schlecht!):\n"
            cat "$OUT"
    fi
fi
# Files that should not have + signs.
list="/etc/hosts.equiv /etc/shosts.equiv /etc/hosts.lpd"
for f in $list ; do
        if [ -s "$f" ] ; then
                awk '{
                        if ($0 ~ /^\+@.*$/)
                                next;
                        if ($0 ~ /^\+.*$/)
                                printf("\nAchtung: Plus Zeichen in den Dateien %s\n", FILENAME);
                }' $f
        fi
done
# .rhosts check
awk -F: '{ print $1 " " $6 }' /etc/passwd |
while read uid homedir; do
        for j in .rhosts .shosts; do
                if [ -s ${homedir}/$j ] ; then
                        rhost=`ls -lcdbg ${homedir}/$j|sed 's/[%\]/_/g'`
			printf "$uid: $rhost\n"
			test -f "$j" && { # still a race, however ...
			    if egrep + ${homedir}/$j > /dev/null ; then
				printf "\t(enthaelt ein Plus (+) Zeichen!)\n"
			    fi
			}
                fi
        done
done > $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe Nutzer mit .rhosts/.shosts Dateien.\n"
        cat "$OUT"
fi
# Check home directories.  Directories should not be owned by someone else
# or writeable.
awk -F: '/^[^+-]/ { print $1 " " $6 }' /etc/passwd | \
while read uid homedir; do
        if [ -d ${homedir}/ ] ; then
                file=`ls -ldbg ${homedir}|sed 's/[%\]/_/g'`
                printf "$uid $file\n"
        fi
done |
awk '$1 != $4 && $4 != "root" \
        { print "Das Heimatverzeichnis von " $1 " gehoert " $4 }
     $2 ~ /^-....w/ \
        { print "Das Heimatverzeichnis von " $1 " ist schreibbar fuer dessen Gruppe." }
     $2 ~ /^-.......w/ \
        { print "Das Heimatverzeichnis von " $1 " ist weltweit schreibbar!" }' > $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe Heimatverzeichnisse.\n"
        sort -u "$OUT"
fi
# Files that should not be owned by someone else or readable.
list=".netrc .rhosts .shosts .Xauthority .pgp/secring.pgp .ssh/identity .ssh/random_seed"
awk -F: '/^[^+-]/ { print $1 " " $6 }' /etc/passwd | \
while read uid homedir; do
        for f in $list ; do
                file=${homedir}/${f}
                if [ -f "$file" ] ; then
                        printf "$uid $f `ls -ldcbg $file`\n"
                fi
        done
done |
awk '$1 != $5 && $5 != "root" \
        { print "Die Datei " $2 " von Nutzer " $1 " gehoert " $5 }
     $3 ~ /^-...r/ \
        { print "Die Datei " $2 " von Nutzer " $1 " hat Leserechte fuer die Gruppe." }
     $3 ~ /^-......r/ \
        { print "Die Datei " $2 " von Nutzer " $1 " ist weltweit lesbar." }
     $3 ~ /^-....w/ \
        { print "Die Datei " $2 " von Nutzer " $1 " hat Schreibrechte fuer die Gruppe." }
     $3 ~ /^-.......w/ \
        { print "Die Datei " $2 " von Nutzer " $1 " hat weltweite Schreibrechte." }' > $OUT
# Files that should not be owned by someone else or writeable.
list=".bashrc .bash_profile .bash_login .bash_logout .cshrc .emacs .exrc \
.forward .klogin .login .logout .profile .tcshrc .fvwmrc .inputrc .kshrc \
.nexrc .screenrc .ssh .ssh/config .ssh/authorized_keys .ssh/environment \
.ssh/known_hosts .ssh/rc .twmrc .xsession .xinitrc .Xdefaults .rhosts \
.shosts .Xauthority .pgp/secring.pgp .ssh/identity .ssh/random_seed \
.pgp/randseed.bin .netrc .exrc .vimrc .viminfo"
awk -F: '/^[^+-]/ { print $1 " " $6 }' /etc/passwd | \
while read uid homedir; do
        for f in $list ; do
                file=${homedir}/${f}
                if [ -f "$file" ] ; then
                        printf "$uid $f `ls -ldcbg $file|sed 's/[%\]/_/g'`\n"
                fi
        done
done |
awk '$1 != $5 && $5 != "root" \
        { print "Die Datei " $2 " von Nutzer " $1 " gehoert " $5 }
     $3 ~ /^-....w/ \
        { print "Die Datei " $2 " von Nutzer " $1 " hat Schreibrechte fuer die Gruppe." }
     $3 ~ /^-.......w/ \
        { print "Die Datei " $2 " von Nutzer " $1 " ist weltweit beschreibbar." }' >> $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe versteckte Dateien.\n"
        sort -u "$OUT"
fi
# Mailboxes should be owned by user and unreadable.
ls -cl /var/spool/mail | sed 1d | \
awk '$3 != $9 \
        { print "Die Mailbox von " $9 " gehoert " $3 }
     $1 != "-rw-------" \
        { print "Die Mailbox " $9 " hat die Rechte " $1 " und gehoert zur Gruppe " $4 }' > $OUT
if [ -s "$OUT" ] ; then
        printf "\nUeberpruefe Eigentumsverhaeltnisse der Mailboxen.\n"
        sort -u "$OUT"
fi
# File systems should not be globally exported.
if [ -s /etc/exports ] ; then
        awk '{
                if (($1 ~ /^#/) || ($1 ~ /^$/))
                        next;
                readonly = 0;
                for (i = 2; i <= NF; ++i) {
                        if ($i ~ /^-ro$/)
                                readonly = 1;
                        else if ($i !~ /^-/)
                                next;
                }
                if (readonly)
                        print "Dateisystem " $1 " wird weltweit read-only exportiert.";
		else
                        print "Dateisystem " $1 " wird weltweit read-write exportiert.";
        }' < /etc/exports > $OUT
        if [ -s "$OUT" ] ; then
                printf "\nUeberprufe exportierte Dateisysteme.\n"
                cat "$OUT"
        fi
fi
# promisc check
> $OUT
/sbin/ifconfig|/usr/bin/grep PROMISC 1> /dev/null && /sbin/ifconfig|\
	/usr/bin/grep -C PROMISC | grep -v '   [RT]X p'> $OUT
if [ -s "$OUT" ] ; then
    printf "\nUeberpruefe Schnittstellen auf Lauschmodus.\n"
    cat "$OUT"
fi
# list loaded modules
> $OUT
test -e /proc/modules && { lsmod 2> /dev/null | grep -v '^Module .* Used by$' | awk '{print$1}' | sort > $OUT
 if [ -s "$OUT" ] ; then
    printf "\nFolgende ladbare Kernel-Module sind zur Zeit installiert:\n"
    cat "$OUT"
 fi
}
# nfs mounts with missing nosuid
> $OUT
/bin/mount | /usr/bin/grep -v nosuid | /usr/bin/grep ' nfs ' |sort > $OUT
if [ -s "$OUT" ] ; then
    printf "\nDie folgenden per NFS eingebundenen Dateisysteme haben die nosuid Option gesetzt:\n"
    cat "$OUT"
fi
# display programs with bound sockets
if [ -x /usr/bin/lsof ]; then
    printf "\nFolgende Programme haben sich mit den genannten Anschluessen verbunden:\n"
    /usr/bin/lsof -i -n -P | egrep 'UDP|TCP.*LISTEN' | sed 's/....[0-9]u  IP.*     /   /' | sed 's/  FD   TYPE DEVICE SIZE NODE NAME/PROTO PORT/' | sed 's/ [0-9][0-9]* / /'|sed 's/ PID / /'|sort -u
fi

####
#
# Cleaning up
#
rm -rf "$TMPDIR"
exit 0
# END OF SCRIPT
